/**
 * Stores, reads and alters Kobold Adventure settings.
 */
function Settings() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var settingsSlot = "settings";

    var flatChest = "flatchest";
    var hotLoad = "hotload";
    var oldLoad = "oldLoad";
    var hideNsfw = "hidensfw";
    var saveScroll = "saveScroll";
    var scrollBottom = "scrollBottom";
    var audio = "audioEnabled";
    var audioInMenu = "audioInMenu";
    var masterVolume = "masterVolume";
    var effectsVolume = "effectsVolume";
    var backgroundVolume = "backgroundVolume";

    // Master listener for setting changes.
    var masterListener = new SettingMasterListener(settingsSlot);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Gets the setting with the passed key.
     * @param {string} key The key of the setting.
     * @returns {?} The setting.
     */
    function getSetting(key) {
        var json = localStorage.getItem(settingsSlot) || "{}";
        var obj = JSON.parse(json);
        return obj[key];
    }

    /**
     * Sets the setting with the passed key to the passed value.
     * @param {string} key The key of the setting.
     * @param {?} value The value to set the setting to.
     */
    function setSetting(key, value) {
        var json = localStorage.getItem(settingsSlot) || "{}";
        var obj = JSON.parse(json);
        obj[key] = value;
        localStorage.setItem(settingsSlot, JSON.stringify(obj));
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Gets the flat chested female setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getFlatChestedFemale = function () {
        return getSetting(flatChest);
    };

    /**
     * Sets the flat chested female setting.
     * @param {boolean} value The value to set it to.
     */
    this.setFlatChestedFemale = function (value) {
        setSetting(flatChest, value);
    };

    /**
     * Gets the hotload setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getHotLoad = function () {
        return getSetting(hotLoad);
    };

    /**
     * Sets the hotload setting.
     * @param {boolean} value The value to set it to.
     */
    this.setHotLoad = function (value) {
        setSetting(hotLoad, value);
    };

    /**
     * Gets the old load setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getOldLoad = function () {
        return getSetting(oldLoad);
    };

    /**
     * Sets the old load setting.
     * @param {boolean} value The value to set it to.
     */
    this.setOldLoad = function (value) {
        setSetting(oldLoad, value);
    };

    /**
     * Gets the hide NSFW setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getHideNsfw = function () {
        return getSetting(hideNsfw);
    };

    /**
     * Sets the hide NSFW setting.
     * @param {boolean} value The value to set it to.
     */
    this.setHideNsfw = function (value) {
        setSetting(hideNsfw, value);
    };

    /**
     * Gets the save scroll setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getSaveScroll = function () {
        return getSetting(saveScroll);
    };

    /**
     * Sets the save scroll setting.
     * @param {boolean} value The value to set it to.
     */
    this.setSaveScroll = function (value) {
        setSetting(saveScroll, value);
    };

    /**
     * Gets the scroll bottom setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getScrollBottom = function () {
        return getSetting(scrollBottom);
    };

    /**
     * Sets the scroll bottom setting.
     * @param {boolean} value The value to set it to.
     */
    this.setScrollBottom = function (value) {
        setSetting(scrollBottom, value);
    };

    /**
     * Gets the audio setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getAudioEnabled = function () {
        return getSetting(audio);
    };

    /**
     * Sets the audio setting.
     * @param {boolean} value The value to set it to.
     */
    this.setAudioEnabled = function (value) {
        setSetting(audio, value);
    };

    /**
     * Gets the audio in menu setting.
     * @returns {undefined | boolean} May return undefined, true or false depending on whether the setting has been set or not. Undefined is to be treated as false.
     */
    this.getAudioInMenu = function () {
        return getSetting(audioInMenu);
    };

    /**
     * Sets the audio in menu setting.
     * @param {boolean} value The value to set it to.
     */
    this.setAudioInMenu = function (value) {
        setSetting(audioInMenu, value);
    };

    /**
     * Transforms the setting from its raw JSON-converted value to a usable value.
     * @param {int} setting The setting before conversion.
     * @returns {int} The setting after conversion.
     */
    function volumeTransform(setting) {
        return setting ? setting - 1 : 50;
    }

    /**
     * Gets the master volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getMasterVolume = function () {
        var setting = getSetting(masterVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the master volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setMasterVolume = function (value) {
        setSetting(masterVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.masterVolumeListener = new SettingSubListener(masterListener, masterVolume, volumeTransform);

    /**
     * Gets the effects volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getEffectsVolume = function () {
        var setting = getSetting(effectsVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the effects volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setEffectsVolume = function (value) {
        setSetting(effectsVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.effectsVolumeListener = new SettingSubListener(masterListener, effectsVolume, volumeTransform);

    /**
     * Gets the background volume setting.
     * The volume is internally converted to its value +1.
     * It is automatically converted back to its usual 0-100 format.
     * @returns {number} The master volume. May be any number from 0 up to and including 100. Default 50.
     */
    this.getBackgroundVolume = function () {
        var setting = getSetting(backgroundVolume);
        return volumeTransform(setting);
    };

    /**
     * Sets the background volume setting.
     * The volume will internally be set to the passed value +1, to ease checking whether or not it has been set.
     * @param {number} value The value to set it to.
     */
    this.setBackgroundVolume = function (value) {
        setSetting(backgroundVolume, +value + 1);
    };

    /**
     * Setting listener for the volume.
     */
    this.backgroundVolumeListener = new SettingSubListener(masterListener, backgroundVolume, volumeTransform);
}

